# Cleaning
import re
from Sastrawi.Stemmer.StemmerFactory import StemmerFactory
from Sastrawi.StopWordRemover.StopWordRemoverFactory import StopWordRemoverFactory
from nltk import word_tokenize
import pandas as pd


class TextCleaningIndo:
    def basic_clean(self, data):
        self.data = data
        # case folding
        self.data = self.data.lower()
        # remove punctuation
        self.data = re.sub('[^\w\s]', ' ', self.data)
        # remove numbers
        self.data = re.sub('\d+', '', self.data)
        # remove extra whitespace
        self.data = ' '.join(self.data.split())
        # remove emoji
        self.data = re.sub(r'[^\x00-\x7F]+', ' ', self.data)
        # remove new line
        self.data = re.sub('\n', ' ', self.data)
        return self.data

    # Stemming
    def stemmer_sastrawi(self, data):
        self.data = data
        factory = StemmerFactory()
        stemmer = factory.create_stemmer()
        self.data = stemmer.stem(self.data)
        return self.data

    # Stopword Removal
    def stopword_removal(self, data):
        self.data = data
        factory = StopWordRemoverFactory()
        stopword = factory.create_stop_word_remover()
        self.data = stopword.remove(self.data)
        return self.data

    def formalize_text(self, data):
        self.data = data
        # Kamus alay
        kamus_alay_1 = pd.read_csv(
            "https://raw.githubusercontent.com/ramaprakoso/analisis-sentimen/master/kamus/kbba.txt",
            delimiter="\t",
            header=None,
            names=['slang', 'formal'])
        kamus_alay_2 = pd.read_csv(
            "https://raw.githubusercontent.com/nasalsabila/kamus-alay/master/colloquial-indonesian-lexicon.csv",
            usecols=["slang", "formal"])

        kamus_alay = pd.concat([kamus_alay_1, kamus_alay_2])

        # Dictionary bahasa alay
        dict_alay = dict()
        for index, row in kamus_alay.iterrows():
            dict_alay[row['slang']] = row['formal']

        word_tokens = word_tokenize(self.data)
        result = [dict_alay.get(x, x) for x in word_tokens]
        return ' '.join(result)

    def all_preprocessing(self, data):
        self.data = data
        self.data = self.basic_clean(self.data)
        self.data = self.formalize_text(self.data)
        self.data = self.stemmer_sastrawi(self.data)
        self.data = self.stopword_removal(self.data)
        return self.data



from sklearn.model_selection import GridSearchCV
from sklearn.model_selection import RandomizedSearchCV

class AutoSearchCV:

    def __init__(self, model, scoring=None, cv=5, n_jobs=None, verbose=1,type="grid"):
        self.model = model
        self.scoring = scoring
        self.cv = cv
        self.n_jobs = n_jobs
        self.verbose = verbose
        self.type = type


    # TODO: Review params combinations
    def params_dict(self):

        # Regression
        if str(self.model) == "LinearRegression()":
            return {
                    "normalize": [True, False]
                    }

        elif str(self.model) == "Ridge()":
            return {"alpha": [0.01, 0.1, 1, 10, 100, 1000],
                    "solver": ["sag", "saga", "lsqr", "svd", "cholesky"]
                    }

        elif str(self.model) == "Lasso()":
            return {"alpha": [0.01, 0.1, 1, 10, 100, 1000],
                    "solver": ["sag", "saga", "lsqr", "svd", "cholesky"]
                    }
        
        elif str(self.model) == "XGBRegressor()":
            return {"learning_rate": [0.01, 0.1, 1, 10, 100, 1000],
                    "max_depth": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    "n_estimators": [10, 20, 30, 40, 50, 60, 70, 80, 90, 100],
                    "gamma": [0, 0.1, 1, 10, 100, 1000],
                    "min_child_weight": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    "subsample": [0.5, 0.6, 0.7, 0.8, 0.9, 1.0],
                    }
        
        elif str(self.model) == "LGBMRegressor()":
            return {"learning_rate": [0.01, 0.1, 1, 10, 100, 1000],
                    "max_depth": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    "n_estimators": [10, 20, 30, 40, 50, 60, 70, 80, 90, 100],
                    "gamma": [0, 0.1, 1, 10, 100, 1000],
                    "min_child_weight": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    "subsample": [0.5, 0.6, 0.7, 0.8, 0.9, 1.0],
                    }
        
        elif str(self.model) == "SVR()":
            return {"C": [0.01, 0.1, 1, 10, 100, 1000],
                    "kernel": ["linear", "poly", "rbf", "sigmoid"],
                    "degree": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    "gamma": [0, 0.1, 1, 10, 100, 1000],
                    "coef0": [0, 0.1, 1, 10, 100, 1000],
                    }
        
        elif str(self.model) == "RandomForestRegressor()":
            return {"n_estimators": [10, 20, 30, 40, 50, 60, 70, 80, 90, 100],
                    "max_depth": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    "max_features": ["auto", "sqrt", "log2"],
                    "min_samples_split": [2, 5, 10],
                    "min_samples_leaf": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    }
        
        elif str(self.model) == "GradientBoostingRegressor()":
            return {"learning_rate": [0.01, 0.1, 1, 10, 100, 1000],
                    "n_estimators": [10, 20, 30, 40, 50, 60, 70, 80, 90, 100],
                    "max_depth": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    "max_features": ["auto", "sqrt", "log2"],
                    "min_samples_split": [2, 5, 10],
                    "min_samples_leaf": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    }
        
        elif str(self.model) == "AdaBoostRegressor()":
            return {"n_estimators": [10, 20, 30, 40, 50, 60, 70, 80, 90, 100],
                    "learning_rate": [0.01, 0.1, 1, 10, 100, 1000],
                    "loss": ["linear", "square", "exponential"],
                    }
        
        elif str(self.model) == "BaggingRegressor()":
            return {"n_estimators": [10, 20, 30, 40, 50, 60, 70, 80, 90, 100],
                    "max_samples": [0.5, 0.6, 0.7, 0.8, 0.9, 1.0],
                    "max_features": ["auto", "sqrt", "log2"],
                    }
        
        elif str(self.model) == "ExtraTreesRegressor()":
            return {"n_estimators": [10, 20, 30, 40, 50, 60, 70, 80, 90, 100],
                    "max_depth": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    "max_features": ["auto", "sqrt", "log2"],
                    "min_samples_split": [2, 5, 10],
                    "min_samples_leaf": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    }
        
        elif str(self.model) == "VotingRegressor()":
            raise ValueError("VotingRegressor is not supported")

        # Classification model
        elif str(self.model) == "LogisticRegression()":
            return {"penalty": ["l1", "l2"],
                    "C": [0.01, 0.1, 1, 10, 100, 1000],
                    "solver": ["liblinear", "sag", "saga"]
                    }
        
        elif str(self.model) == "SGDClassifier()":
            return {"loss": ["hinge", "log", "modified_huber", "squared_hinge", "perceptron"],
                    "penalty": ["l2", "l1", "elasticnet"],
                    "alpha": [0.01, 0.1, 1, 10, 100, 1000],
                    "learning_rate": ["constant", "optimal", "invscaling", "adaptive"],
                    "eta0": [0.01, 0.1, 1, 10, 100, 1000],
                    "power_t": [0.01, 0.1, 1, 10, 100, 1000],
                    "tol": [0.01, 0.1, 1, 10, 100, 1000],
                    "warm_start": [True, False],
                    "class_weight": ["balanced", None],
                    }
        
        elif str(self.model) == "SVC()":
            return {"C": [0.01, 0.1, 1, 10, 100, 1000],
                    "kernel": ["linear", "poly", "rbf", "sigmoid"],
                    "degree": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    "gamma": [0, 0.1, 1, 10, 100, 1000],
                    "coef0": [0, 0.1, 1, 10, 100, 1000]
                    }
                
        elif str(self.model) == "RandomForestClassifier()":
            return {"n_estimators": [10, 20, 30, 40, 50, 60, 70, 80, 90, 100],
                    "max_depth": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    "max_features": ["auto", "sqrt", "log2"],
                    "min_samples_split": [2, 5, 10],
                    "min_samples_leaf": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    }
        
        elif str(self.model) == "GradientBoostingClassifier()":
            return {"learning_rate": [0.01, 0.1, 1, 10, 100, 1000],
                    "n_estimators": [10, 20, 30, 40, 50, 60, 70, 80, 90, 100],
                    "max_depth": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    "max_features": ["auto", "sqrt", "log2"],
                    "min_samples_split": [2, 5, 10],
                    "min_samples_leaf": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    }
        
        elif str(self.model) == "AdaBoostClassifier()":
            return {"n_estimators": [10, 20, 30, 40, 50, 60, 70, 80, 90, 100],
                    "learning_rate": [0.01, 0.1, 1, 10, 100, 1000],
                    "loss": ["linear", "square", "exponential"],
                    }
        
        elif str(self.model) == "BaggingClassifier()":
            return {"n_estimators": [10, 20, 30, 40, 50, 60, 70, 80, 90, 100],
                    "max_samples": [0.5, 0.6, 0.7, 0.8, 0.9, 1.0],
                    "max_features": ["auto", "sqrt", "log2"],
                    }
        
        elif str(self.model) == "ExtraTreesClassifier()":
            return {"n_estimators": [10, 20, 30, 40, 50, 60, 70, 80, 90, 100],
                    "max_depth": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    "max_features": ["auto", "sqrt", "log2"],
                    "min_samples_split": [2, 5, 10],
                    "min_samples_leaf": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    }
        
        elif str(self.model) == "VotingClassifier()":
            raise ValueError("VotingClassifier is not supported")
        
        elif str(self.model) == "XGBClassifier()":
            return {"n_estimators": [10, 20, 30, 40, 50, 60, 70, 80, 90, 100],
                    "max_depth": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    "learning_rate": [0.01, 0.1, 1, 10, 100, 1000],
                    "gamma": [0, 0.1, 1, 10, 100, 1000],
                    "min_child_weight": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    "max_delta_step": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    "subsample": [0.5, 0.6, 0.7, 0.8, 0.9, 1.0],
                    }
        
        elif str(self.model) == "LGBMClassifier()":
            return {"n_estimators": [10, 20, 30, 40, 50, 60, 70, 80, 90, 100],
                    "learning_rate": [0.01, 0.1, 1, 10, 100, 1000],
                    "num_leaves": [10, 20, 30, 40, 50, 60, 70, 80, 90, 100],
                    "max_depth": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    "min_child_samples": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    "min_child_weight": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    "subsample": [0.5, 0.6, 0.7, 0.8, 0.9, 1.0],
                    }
        
        else:
            raise ValueError("Estimator not supported")


    def search(self, X, y):
        self.X = X
        self.y = y
        
        if self.type == "grid":
            self.grid = GridSearchCV(self.model, self.params_dict(), scoring=self.scoring, cv=self.cv, n_jobs=self.n_jobs,
                                verbose=self.verbose)
            self.grid.fit(self.X, self.y)
            self.best_estimator_ = self.grid.best_estimator_
            self.best_score_ = self.grid.best_score_
            self.best_params_ = self.grid.best_params_
            self.best_index_ = self.grid.best_index_
            self.cv_results_ = self.grid.cv_results_
        
        elif self.type == "random":
            self.random = RandomizedSearchCV(self.model, self.params_dict(), scoring=self.scoring, cv=self.cv, n_jobs=self.n_jobs, verbose=self.verbose)
            self.random.fit(self.X, self.y)
            self.best_estimator_ = self.random.best_estimator_
            self.best_score_ = self.random.best_score_
            self.best_params_ = self.random.best_params_
            self.best_index_ = self.random.best_index_
            self.cv_results_ = self.random.cv_results_

        else:
            raise ValueError("Type not supported: Please choose 'grid' or 'random'")
        
        print(f"""
        
        SEARCH SUMMARY
        
        Best estimator  : {self.best_estimator_}
        Best score      : {self.best_score_}
        Best params     : {self.best_params_}
        
        """)


from sklearn.linear_model import LinearRegression
from sklearn.linear_model import Ridge
from sklearn.linear_model import Lasso
from xgboost import XGBRegressor
from lightgbm import LGBMRegressor
from sklearn.svm import SVR
from sklearn.ensemble import RandomForestRegressor
from sklearn.ensemble import GradientBoostingRegressor
from sklearn.ensemble import AdaBoostRegressor
from sklearn.ensemble import BaggingRegressor
from sklearn.ensemble import ExtraTreesRegressor
from sklearn.ensemble import VotingRegressor

# Classification model
from sklearn.linear_model import LogisticRegression
from sklearn.linear_model import SGDClassifier
from sklearn.svm import SVC
from sklearn.ensemble import RandomForestClassifier
from sklearn.ensemble import GradientBoostingClassifier
from sklearn.ensemble import AdaBoostClassifier
from sklearn.ensemble import BaggingClassifier
from sklearn.ensemble import ExtraTreesClassifier
from sklearn.ensemble import VotingClassifier
from sklearn.neighbors import KNeighborsClassifier
from xgboost import XGBClassifier
from lightgbm import LGBMClassifier

# Metrics
from sklearn.metrics import mean_squared_error
from sklearn.metrics import f1_score

# Misc
import numpy as np
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
import warnings
warnings.filterwarnings('ignore')


class BaselineModel:

    def __init__(self, type="regression"):
        self.type = type

    def regression_models(self):
        return [
            ("LinearRegression", LinearRegression()),
            ("RidgeRegression", Ridge()),
            ("LassoRegression", Lasso()),
            ("XGBRegressor", XGBRegressor()),
            ("LGBMRegressor", LGBMRegressor()),
            ("SupportVectorRegressor", SVR()),
            ("RandomForestRegressor", RandomForestRegressor()),
            ("GradientBoostingRegressor", GradientBoostingRegressor()),
            ("AdaBoostRegressor", AdaBoostRegressor()),
            ("BaggingRegressor", BaggingRegressor()),
            ("ExtraTreesRegressor", ExtraTreesRegressor()),
            ("VotingRegressor", VotingRegressor(estimators=[("LinearRegression", LinearRegression()),
                                                            ("Ridge", Ridge()),
                                                            ("Lasso", Lasso()),
                                                            ("XGBRegressor",
                                                             XGBRegressor()),
                                                            ("LGBMRegressor",
                                                             LGBMRegressor()),
                                                            ("SVR", SVR()),
                                                            ("RandomForestRegressor",
                                                             RandomForestRegressor()),
                                                            ("GradientBoostingRegressor",
                                                             GradientBoostingRegressor()),
                                                            ("AdaBoostRegressor",
                                                             AdaBoostRegressor()),
                                                            ("BaggingRegressor",
                                                             BaggingRegressor()),
                                                            ("ExtraTreesRegressor",
                                                             ExtraTreesRegressor())
                                                            ]
                                                )
             )
        ]

    def classification_models(self):
        return [
            ("LogisticRegression", LogisticRegression()),
            ("SGDClassifier", SGDClassifier()),
            ("SupportVectorClassifier", SVC()),
            ("RandomForestClassifier", RandomForestClassifier()),
            ("GradientBoostingClassifier", GradientBoostingClassifier()),
            ("AdaBoostClassifier", AdaBoostClassifier()),
            ("BaggingClassifier", BaggingClassifier()),
            ("ExtraTreesClassifier", ExtraTreesClassifier()),
            ("XGBClassifier", XGBClassifier(verbosity=0, silent=True)),
            ("LGBMClassifier", LGBMClassifier()),
            ("KNeighborsClassifier", KNeighborsClassifier()),
            ("VotingClassifier", VotingClassifier(estimators=[("LogisticRegression", LogisticRegression()),
                                                              ("SGDClassifier",
                                                               SGDClassifier()),
                                                              ("SVC", SVC()),
                                                              ("RandomForestClassifier",
                                                               RandomForestClassifier()),
                                                              ("GradientBoostingClassifier",
                                                               GradientBoostingClassifier()),
                                                              ("AdaBoostClassifier",
                                                               AdaBoostClassifier()),
                                                              ("BaggingClassifier",
                                                               BaggingClassifier()),
                                                              ("ExtraTreesClassifier",
                                                               ExtraTreesClassifier()),
                                                              ("XGBClassifier",
                                                               XGBClassifier(verbosity=0, silent=True)),
                                                              ("LGBMClassifier",
                                                               LGBMClassifier()),
                                                              ("KNeighborsClassifier",
                                                               KNeighborsClassifier())
                                                              ]
                                                  )
             )
        ]

    def get_models(self):
        if self.type == "regression":
            return self.regression_models()
        elif self.type == "classification":
            return self.classification_models()
        else:
            raise ValueError("Type must be regression or classification")

    def score(self, y_true, y_pred):
        if self.type == "regression":
            return np.sqrt(mean_squared_error(y_true, y_pred))
        elif self.type == "classification":
            return f1_score(y_true, y_pred, average="macro")
        else:
            raise ValueError("Type must be regression or classification")

    def evaluate(self, X_train, X_test, y_train, y_test):
        self.test_result = []
        for model in self.get_models():
            model[1].fit(X_train, y_train)
            y_pred = model[1].predict(X_test)
            if self.type == "regression":
                self.test_result.append([model[0], self.score(y_test, y_pred), model[1]])

            elif self.type == "classification":
                self.test_result.append([model[0], self.score(y_test, y_pred), model[1]])

        if self.type == "regression":
            self.test_result = pd.DataFrame(
                self.test_result, columns=["model", "rmse_score", "model class"]).sort_values(by="rmse_score")

        elif self.type == "classification":
            self.test_result = pd.DataFrame(
                self.test_result, columns=["model", "f1_score", "model_class"]).sort_values(by="f1_score", ascending=False)
        
        self.best_model = self.test_result.iloc[0, 2]

    def plot_baseline(self):
        if self.type == "regression":
            plt.title("Regression Baseline Model")
            # plt.xticks(rotation=90)
            sns.barplot(y="model", x="rmse_score", data=self.test_result)
            plt.xlabel("RMSE Score")
            plt.ylabel("Models")
            plt.show()

        elif self.type == "classification":
            plt.title("Classification Baseline Model")
            # plt.xticks(rotation=90)
            sns.barplot(y="model", x="f1_score", data=self.test_result)
            plt.xlabel("F1 Score")
            plt.ylabel("Models")
            plt.show()



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name='shortcutml',
    packages=['shortcutml.model_selection',
              'shortcutml.preprocessing', ],
    package_dir={
        "shortcutml": "./shortcutml",
    },
    version='0.5',

    license='MIT',
    description='Machine learning baseline prototyping tools',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Sulthan Abiyyu Hakim',
    author_email='sabiyyuhakim@student.ub.ac.id',
    url='https://github.com/SulthanAbiyyu/ShortcutML',

    download_url='https://github.com/SulthanAbiyyu/ShortcutML/archive/refs/tags/0.5.tar.gz',

    keywords=['machine learning', 'summary'],
    install_requires=[
        "joblib",
        "lightgbm",
        "matplotlib",
        "nltk",
        "numpy",
        "pandas",
        "PySastrawi",
        "scikit-learn",
        "seaborn",
        "xgboost"
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],

)
